########################################################################
## Useful functions
########################################################################
alive<-function(at=2400,opop){
  ## returns a T/F vector useful for selecting the rows of opop of
  ## only people alive at "at"
  
  return(opop$dob <= at & (opop$dod >= at | opop$dod ==0))
  ##  return(opop[opop$dob <= at & (opop$dod >= at | opop$dod ==0),])
}

read.socrates<-function(file="../HMDdata/Sweden/hmdMort1950"){
  ## reads a socsim ratefile and returns the rate blocks as list object
  ## 
  
  socin<-read.delim(file=file,sep=" ",comment='*',as.is=T,header=F,fill=T)

  ## find the socsim rate designation
  delims<-grep(pattern='death|birth|marriage|divorce',x=socin$V1)

  delims<-c(delims,dim(socin)[1]+1) ## need the last block
  rateKey<-NULL
  result<-list()
  for(d in 1:(length(delims)-1)){
    rateKey<-c(rateKey,paste(socin[delims[d],],collapse='.'))
    rateblock<-data.matrix(socin[(delims[d]+1):(delims[d+1]-1),1:3])
    colnames(rateblock)<-c("yr","mo","rate")
    dframe<-data.frame(rateblock)
    result[[d]]<-dframe
  }
  names(result)<-rateKey
  return(result)
}
  
########################################################################

asYr<-function(x,lastmo=endmo){
  ## convert sim month to a real calendar year
  ## handy for graphing mainly.
  ## requires that FinalSimYear be set in the GlobalEnv
  stopifnot("FinalSimYear" %in% objects(pos=1))
  yr<-ifelse(x==lastmo,FinalSimYear,
             FinalSimYear - trunc((lastmo -x )/12))
  return(yr)
}
########################################################################
zna<-function(x){return(ifelse(x==0,NA,x))}

########################################################################
dec<-function(year){
  ## returns simulation month equiv of dec of the given real calendar year
  ## requires that FinalSimYear be set in the GlobalEnv
  ## assumes that simulation ends Dec 31
  stopifnot("FinalSimYear" %in% objects(pos=1))
  return( endmo - ((FinalSimYear - year)*12))
}
########################################################################
lx_from_nmx <- function(Nmx){
  ## computes the probability of dying using standard lifetable formulas
  ## nqx= [n*nMX]/[1+ (n-nax)*nMx)] Ken's book pg 158
  ## for use in reproducing Josh's isoplot
  nmx<- 1-(1-Nmx)^12
  nqx <- (nmx*1)/(1+(0.5*nmx))
  ##now correct for the open ended interval, so that inf_q_x =1
  nqx[length(nqx)]<- 1
  
  nlx <- rep(NA,(length(nqx)+1))
  nlx[1] <- 1
  for (jj in 1:(length(nlx)-1)){
    nlx[jj+1]<- nlx[jj]*(1-nqx[jj])
  }
  
  return(nlx)
}
########################################################################

median.age<-function(x){
  ## finds a median assuming that NA's are oldest. Generally they
  ## are in fact the not-dead-yets. Depending when simulation ends
  ## this is problematic.
  sort(x,na.last=T)[trunc(length(x))/2]
}

########################################################################
