########################################################################
## # # # Constants
########################################################################

## ### ## These may change if you fiddle with the .sup file!!

FinalSimYear<-2145  ## check .sup file and give this careful consideration

endmo<-6541  #5307  ## last month of simulation See Socsim output

EndYr<-endmo:(endmo-11)


########################################################################
## Useful functions
########################################################################
source("Macros.r")
##pdf(file="sandwich.pdf")
########################################################################
## Read opop and omar files and assign usual names to columns
########################################################################
## read .opop into dataframe
opop<-read.table(file="../SimResults/sandwich.opop",header=F,as.is=T)
## assign names to columns
names(opop)<-c("pid","fem","group",
               "nev","dob","mom","pop","nesibm","nesibp",
               "lborn","marid","mstat","dod","fmult")
## handy for selecting by pid and safer than relying on pid==row, a
## condition that is generally but not guaranteed to always obtain.
rownames(opop)<-opop$pid

## Do we need the marriage file?
##omar<-read.table(file="../SimResults/sandwich.omar",header=F,as.is=T)
##names(omar)<-c("mid","wpid","hpid","dstart","dend",
##               "rend","wprior","hprior")
   
##rownames(omar)<-omar$mid

########################################################################
## Early exploration
########################################################################

## Over the 20th and 21st century there have and will be significant
## changes in fertility and mortality.  Some interesting numbers for our
## sandwich investigation are the average age at child bearing and the
## expected years of life remaining at birth of grandchild.
##
## To calculate these numbers from a lifetable requires ignorning the
## changes in rates over time and the resultant distribution of
## maternal age at birth of average child.


## Average age of grandmother at birth of average child, Mu2 for short.

# mother's and grandmother's age at birth of typical child
opop$Mage<- opop$dob- opop$dob[zna(opop$mom)]
opop$GMage<- opop$dob- opop$dob[ zna(opop$mom[ zna(opop$mom)])]


Mu<-tapply(opop$Mage,asYr(opop$dob),mean)/12
Mu2<-tapply(opop$GMage,asYr(opop$dob),mean)/12

## plot the averages over time
##pdf(file="meanage.pdf")
plot(range(Mu,Mu2,na.rm=T)~c(1900,2100),type='n',
     main="Mean age (G)mother at birth of average child",
     xlab="Year", ylab="Mean age")
points(Mu2~names(Mu),xlim=c(1900,2100),cex=.5)
points(Mu~names(Mu),xlim=c(1900,2100),cex=.5)
text(x=1920,y=Mu["1920"],pos=3,label="mother")
text(x=1920,y=Mu2["1920"],pos=3,label="Gmother")
##dev.off()
## Diversion -- one could look up mean age at child bearing in the HFD

##
## Expected years of life remaining at birth of grandchild is the same
## as average age of ego at death of grandmother...right?


## month of mom's death 
opop$Mdeath<- zna(opop[zna(opop$mom),"dod"])  ##NA if mom is still alive
## month of Maternal Grandmother's death
opop$GMdeath<-opop$Mdeath[match(opop$mom, opop$pid)]
## great grandmother?
opop$GGMdeath<-opop$GMdeath[match(opop$mom, opop$pid)]

opop$ageGmomMDeath<- opop$GMdeath - opop$dob
opop$ageGGmomMDeath<- opop$GGMdeath - opop$dob


GmomCoSurvival<-with(opop,
                     tapply(ageGmomMDeath,asYr(dob),mean)/12)

plot(GmomCoSurvival~I(names(GmomCoSurvival)),xlim=c(1900,2050),type='l',
     main="Mean Years of Grandmaternal Life after Birth of Average Child",
     xlab="Birth Cohort",ylab="Years")

GGmomCoSurvival<-with(opop,
                     tapply(ageGGmomMDeath,asYr(dob),mean)/12)

par(new=T)
plot(GGmomCoSurvival~I(names(GGmomCoSurvival)),lty=2,axes=F,type='l',
     xlab="",ylab="",col='blue')
axis(side=4,col='blue')     
text(x=2000,y=GGmomCoSurvival["1950"],col="blue",
     pos=3,label="Great-Grandmother scale on right")

########################################################################
##Assumption 0: Define "sandwich" For now, let's consider it to be the
##condition of a child < 10 years old and his/her grandmother within 5
##years of death
########################################################################

########################################################################
## Exercise 1:  From the mother's point of view 
## Count months spent with young children *and* mother < 5 yrs from death
## that is months of the Mother's life spent in a state of sandwichedness
########################################################################

## Some useful additional variables

## last born's 10th birthday or death date BUT late in the simulation
## some kids may survive past 2150 so take those out.
opop$lc.10<- ifelse(opop[zna(opop$lborn),"dod"] != 0,
                    pmin(opop[zna(opop$lborn),"dob"] + (10*12),
                         opop[zna(opop$lborn),"dod"]),
                    opop[zna(opop$lborn),"dob"] + (10*12)
                    )

                    ## or death

## thanks Bruno and Romesh for pointing out that this lc.10 misses
## months when lborn dies BUT next born is NOT yet 10
## and for showing that it makes no difference in the US context
## construct first born's dob
## collect first born children
fborns<-opop[opop$nesibm == 0,]
## match to mom and grab date of birth
opop$fb.0<- fborns[match(opop$pid,fborns$mom),"dob"] 

## calculate duration spent sandwiched:  From mother's point of view

## End of Sandwich is the earlier of:
##   (last kid's 10th Birthday, Mom's death)
## Beginning of Sandwich is the later of:
## (first birth and 5years before mom's death)

##Granda is the mom of the mother that we are considering.

## There is NO sandwich when:

## 1.Grandma's last 5 years of life start
## after the youngest maternal grandchildren is older than 10.  That
## is if the date of grandma's death minus 5 years is bigger than the
## date when the youngest grandchildren is 10.  Example: Youngest
## granchildren was born in 1980 and grandma dies in 2010

## OR 2.The
## prospective granmda dies before any of the prospective maternal
## grandchildren were born.  That is, the date of grandma's death is
## smaller than th birthdate of the oldest grandchild.  Example:
## Grandma dies in 1970 and the first grandchild was born in 1975.

## In the other situations, the sandwich time can potentially be up to
## 5 years, given our definition of sandwich.  It is given by the
## difference between two quantities: A. The minimum between the year
## of grandma's death and the and the 10th birthday of the youngest
## grandchild.  B. The maximum between the date of birth of the oldest
## grandchild and the grandma's date of death minus 5 years.

## The sandwich time is equal to ( A - B )
## Example 1:
## --1975----------1980------------------1990----------1992
##  birth          birth                youngest       Grandma           
## oldest         youngest              grandchild     dies
## grand child    grandchild            10th B-day     =Mdeath
## =fb.0                                =lc.10         (her in years)
##
##Sandwich time is equal to A-B= 1990 - (1992-5)= 1990-1987= 3 years

## Example 2:
## --1975----------1980-------------1984--------------1990----
##  birth          birth            Grandma         youngest                 
## oldest         youngest          dies            grandchild     
## grand child    grandchild        =Mdeath         10th B-day     
## =fb.0                            (here in years)   =lc.10         
##
##Sandwich time is equal to A- B = 1984 - (1984-5) = 5 years of time


opop$sdw5<- with(opop,
                 ifelse(((Mdeath - 60) > lc.10) |
                        (Mdeath < fb.0) | is.na(fb.0),
                        0,
                         pmin(lc.10,Mdeath) -  pmax(fb.0,(Mdeath-60))   
                        )
                 )

## Question: That was easy, what's wrong with the above calculation?
##
##
## HINT: What happens if a woman has two births more than 10 years apart?


## Example 3. Consider the case of a mother who has only two births, but  more than 10 years apart

## --1975-----------------------1988--------------1991----
##  birth                      Grandma         younger                 
## older                       dies            grandchild     
## grand child                 =Mdeath         birth     
## =fb.0                    (here in years)            
##
##According to our technique, A- B = 1988 - (1988-5) = 5 years of
##sandwiched time. HOWEVER, in truth the sandwiched time is only 2
##years.


## Let's fix those boringly  the calculation is slow so let's only
## do it for those who need the correction. You can close your eyes
## and execute the next few lines. They are neither elegant nor
## interesting. They simply find women with births more then 10 years
## apart and *if* necessary reduce sndw5 as appropriate.  Further, I
## have satisfied myself that the corrections change nothing of
## significance.  But after simulating the data, it will not do to have
## mere "estimates" of the results.

## on the other hand, life is short so for intermediate results
## skip all this by uncommenting the if() below and the } later
##if(){
## <Boring>
## gap between births
gap10<-tapply(opop$dob,opop$mom,function(x){max(diff(sort(x)))})
needfix<-gap10 > 120
needfix[is.infinite(gap10)]<-FALSE

NeedFix<-as.numeric(names(needfix)[needfix])
## if Mdeath is NA gradmother still lives (end of simulation)
NeedFix<-NeedFix[! NeedFix %in% opop$pid[is.na(opop$Mdeath)]]

## Fix can only reduce sandwich so skip if sdw5 is zero
NeedFix<-NeedFix[! NeedFix %in% opop$pid[opop$sdw5 == 0]]

## skip moms born before 1900
NeedFix<-NeedFix[! NeedFix %in% opop$pid[asYr(opop$dob) < 1900]]

## Year of birth of moms with 10 year gaps between children
plot(table(asYr(opop$dob[opop$pid %in% NeedFix])),
     main="Required corrections by year of mother's birth")

## Fix the sdw5 for those moms who need it
## sorry this is is slow if the population is large


sdw5F<-sapply(NeedFix,function(mid){
  kdob<-opop$dob[opop$mom == mid]
  kdod<-opop$dod[opop$mom == mid]
  kdod<-ifelse(kdod == 0,
               endmo,kdod)
  Mdeath<-opop$Mdeath[opop$pid == mid] 
  starts<-pmax(kdob,Mdeath-60)
  ends<-pmin(kdob+120,kdod,Mdeath)
  window<-(Mdeath-60):Mdeath
  names(window)<-window
  for(i in 1:length(kdob)){
    ## remove from window all sandwiched months
    window <- window[! window %in% starts[i]:ends[i]]
  }
  return(60-length(window))
})

names(sdw5F)<-NeedFix
## put in the corrected values.
opop[match(NeedFix,opop$pid),"sdw5"]<-sdw5F[paste(NeedFix)]

##</Boring>
##}  ## uncomment along with if(0) above to skip boring part
########################################################################
## OK WAKE UP NOW -- that boring correction bit is over, now for some
## exciting graphs ...
##
## Let's plot by birth cohort the mean number of months spent in
##sandwich land
########################################################################

## collect means by birth cohort 
sandwich5<-with(opop[opop$fem==1 & !is.na(opop$fb.0),],
                 tapply(sdw5,asYr(dob),mean,na.rm=T))


## some NAs in sandwich5 at begining and end of simulation

## Plot over by Birth Cohort Mother's months as sandwich
##pdf("sand1.pdf")
plot(sandwich5 ~ as.numeric(names(sandwich5)),xlim=c(1900,2050),
     main="Mean months as sandwich for women parity >0\n by mother's birth Cohort",
     xlab="year of birth",ylab="Sandwich Months over lifetime")
lines(supsmu(y=sandwich5,x=as.numeric(names(sandwich5))))

## diversion (1) Note that in the above graph only women of eventual
## parity >0 are  in the denominator.  Perhaps it would be worthwhile
## to draw this plot for all women. In other words to determine the
## effect of parity=0 women.


########################################################################
## Let's graph the percent of mothers who are ever in sandwich state
## again we exclude from consideration, women of parity =0 ... should
## we do that?
########################################################################

sandwich5.ever<-with(opop[opop$fem==1 & !is.na(opop$fb.0),],
                      tapply(sdw5>0,asYr(dob),mean,na.rm=T))

plot(sandwich5.ever ~ as.numeric(names(sandwich5.ever)),xlim=c(1900,2050),
     main="Fraction of women parity>0 who are ever sandwiched",
     xlab="Birth Cohort",ylab="Fraction ever sandwiched")
lines(supsmu(y=sandwich5.ever,x=as.numeric(names(sandwich5.ever))))

##dev.off()

### How do things look in 2050
bc2050<-opop[asYr(opop$dob)==2050,]


## age of woman when mother dies
summary(asYr(bc2050$Mdeath)-2050)
sum(bc2050$Mdeath==0)
summary(bc2050$sdw5)
hist(bc2050$sdw5)

########################################################################
## Exercise 2: Let's consider the coresponding period measure:
## in a given year, what fraction of mothers are sandwiched ?
########################################################################


Years<-seq(1930,2050,by=2)

## ksand5 will be a matrix with one row for each row in opop and
## columns are T/F according to whether or not ego was less the 10
## years old and had maternal Gmom 5 or fewer years from death in
## particular year

ksand5<-NULL


for(Per in Years){
  ## for each year in Years
  Dec<-dec(Per)
  ksand5<-
  cbind(ksand5,
      ifelse(opop$dob < (Dec-131),FALSE, ## > 10  in Jan 
        ifelse(opop$dob > Dec,FALSE,  ## born after dec
          ifelse(opop$dod < (Dec-11),FALSE,## dead before Jan
                    ## kid is under 10 and alive in Jan of Per
            ifelse(opop$GMdeath < (Dec -11),FALSE, ## Gmom dead before Jan
                ifelse((opop$GMdeath - 60) > Dec,FALSE, ## Gmom survives 5+
                      TRUE))))))
}

colnames(ksand5)<-paste(Years)


## Get the mean sandwichedness by year
sandwich5P<-sapply(Years,function(Per){
  res<-tapply(ksand5[,paste(Per)],
              opop$mom,
              sum,na.rm=T)
  ## Which women shall be in the denominator?
  ## this is worth experimenting with
  sel<-opop$fem==1 &              ##women
  opop$dod > (dec(Per)-11) & opop$dob < dec(Per)-(11+18*12) &
  !is.na(opop$dod) &               ## alive and 18 in Jan of year
  !is.na(opop$fb.0) &              ## who are or will be mothers
  (dec(Per) - opop$dob) < (60*12)   ## under 60 yrs old 
  return(mean(res[match(opop$pid[sel],as.numeric(names(res)))]))
})

names(sandwich5P)<-Years

##x11()
plot(sandwich5P~Years,type='b',
     main="Proportion of mothers  sandwiched in calendar year",
     ylab="Proportion sandwiched")

## Diversion (2) well same diversion as 1 really-- in the above graph
## only women who are alive and between 18 and 60  year AND who are
## at some time in their lives parity > 0 are considered at risk.
## Might we want to change that assumption?


########################################################################
## Exercise 3  by cohort over time/age
## Consider the probability of being sandwiched at various ages for
## various birth cohorts.
########################################################################
Cohort<-c(1930,1950,1980)   #seq(1900,1980,by=10)
sandwich5PC<-list()
Years<-as.numeric(colnames(ksand5))
## Get the mean sandwitcedness by year
for (i in 1:length(Cohort)){
  print(c("Cohort",Cohort[i]))
  sandwich5PC[[i]]<-sapply(Years,function(Per,Coh){
    res<-tapply(ksand5[,paste(Per)],
                opop$mom,
                sum,na.rm=T)
    ## Which women shall be in the denominator?
    ## this is worth experimenting with
    sel<-opop$fem==1 &              ##women
    opop$dod > (dec(Per)-11)  &
    !is.na(opop$fb.0) &              ## who are or will be mothers
    asYr(opop$dob) == Coh   ## single year birth cohort of moms
    return(mean(res[match(opop$pid[sel],as.numeric(names(res)))]))
  },Cohort[i])
names(sandwich5PC[[i]])<-Years
}  
names(sandwich5PC)<-Cohort

plt<-colorRampPalette(c("grey50","violetred2"))(length(sandwich5PC))
names(plt)<-names(sandwich5PC)

## graph by calendar year
s5pc<-matrix(unlist(sandwich5PC),ncol=length(Years),byrow=T)
matplot(y=t(s5pc),x=Years,type='l',lwd=2,lty=1,
        col=plt[names(sandwich5PC)])
## Or by age
plot(y=range(unlist(sandwich5PC),na.rm=T),x=c(10,60),type='n',
     main="Age pattern of sandwichedness by cohort of mother",
     xlab="Age of mother",ylab="Proportion sandwiched")
for(coh in names(sandwich5PC)){
  lines(sandwich5PC[[coh]] ~I(as.numeric(names(sandwich5PC[[coh]]))-
        as.numeric(coh)),lwd=2,lty=1,
        col=plt[coh])

    lines(supsmu(y=sandwich5PC[[coh]],
                 x=(as.numeric(names(sandwich5PC[[coh]]))-
                    as.numeric(coh))),
          lwd=.75,lty=2, col=plt[coh])

}
legend(x="topright",fill=plt,legend=names(plt))


## Diversion: who's in the denominator ?
  
########################################################################
## Exercise 4: from the child's point of view: number of months spent
## sandwiching Mom.
########################################################################


opop$sdw5k<-ifelse(pmin(opop$dob+120,opop$GMdeath) -
                pmax(opop$dob, opop$GMdeath -60) >0,
                pmin(opop$dob+120,opop$GMdeath) -
                pmax(opop$dob, opop$GMdeath -60),
                0)

## collect by birth cohort of child
sandwich5.kid<-with(opop,
                 tapply(sdw5k,asYr(dob),mean,na.rm=T))


plot(sandwich5.kid ~ as.numeric(names(sandwich5.kid)),xlim=c(1900,2050),
     main="Mean number of months spent sandwiching mom",
     xlab="Child birth cohort")
  
lines(supsmu(y=sandwich5.kid,x=as.numeric(names(sandwich5.kid))))

##dev.off()
########################################################################
## Extensions

##  (0) Stochasticity-- in any Socsim project it is necessary to run
##  lots of replications in order to washout the stochastic
##  variation. Consider modifying sandwich.r along the lines of
##  the calibration.r to make Socsim run many times with different
##  seeds.  You'll want to concentrate on just one or a few of the
##  numbers that we have calculated above in order to focus on the
##  variation across simulation runs.

## (1)The assumption that grandmothers are sick or in some other way
## burdensome during the last five years before death implies that
## they are perhaps quite healthy and useful before that.  How about
## some measures of time spent with grandma as nanny.

## (2) This simulation uses the UN "Medium" projections for fertility
## and mortality after 2010.  Fertility rates based on the high and
## low scenarios can be found in Socsim-ready form in the UNdata
## directory.  Try snapping one or two of those into place to get an
## idea of the effect of other plausible fertility scenarios.  You
## need only edit the usa.sup file to make this work.


## (3) There are several other countries in the world besides the US. Some
## of those countries are also in the HFD and HMD.  Those countries
## have rates in HMDdata, HFDdata and UNdata. You might wish to copy
## sweden.sup of usa.sup and with some clever editing, create a .sup
## file that will run an even more interesting simulation than this one.

## (4) The changes in sandwichedness are driven by both mortality and
## fertility. Maybe it would be fun to try to decompose the two
## effects.  One approach for doing that could be to run the
## simulations holding the mortality (fertility) rates constant while
## allowing the fertility (mortality) rates to change over time.

## (5) In these simulations we have (for now) set marriage rates to be
## fairly high for young ages and we have set all fertility rates to
## match those provided by the HFD and the UN projections for the
## whole population.  Gathering marriage rates and adjusting fertility
## rates to match is thankless and time consuming.  Consider doing
## that...  OR consider simply making up a  "plausible" stylized set
## of marriage rates and modifying fertility rates accordingly.  Will
## this matter?  If you wish to do this, consider simplifying the
## whole simulation -- maybe just have two segments and run with a
## few sets of marriage/fertility rates to get an idea of the effect
## of including marriage holding everything else constant.  The
## calibration example might be a useful starting point for this.



